; CircuitBread Tutorials
; Microcontroller Basics (PIC10F200), Part 19 - Infrared RGB LED controller
; Link: https://www.circuitbread.com/tutorials/infrared-rgb-led-controller-part-19-microcontroller-basics-pic10f200
; Source code conversion from Microchip MPASM to Microchip XC8 PIC Assembler.

; Include target device specific definitions.
#include <xc.inc>

; Some equates not included in the device specific definitions.
;----- OPTION_REG Bits -----------------------------------------------------
PSA         EQU 0003h
T0CS        EQU 0005h
NOT_GPPU    EQU 0006h

PS0         EQU 0000h
PS1         EQU 0001h
PS2         EQU 0002h

;----- STATUS Bits -----------------------------------------------------
C           EQU 0000h
Z           EQU 0002h

#define DISPLAY         ;Display mode
#undef DISPLAY          ;Uncomment for normal mode

count       EQU 010h    ;Saved value of the timer
byte_count  EQU 011h    ;Counter of processed bytes
bit_count   EQU 012h    ;Counter of processed bits
tw_data     EQU 013h    ;Data to receive/transmit via TWI
port        EQU 014h    ;Helper register to implement 1-wire and TWI
ack         EQU 015h    ;Acknowledgment received from the TWI device
digit       EQU 016h    ;Digit to be decoded for the display
num_l       EQU 017h    ;Lower nibble of the number
num_h       EQU 018h    ;Higher nibble of the number
color       EQU 019h    ;Stores the color of the RGB LED
ir_data     EQU 01Ah    ;First IR byte

#ifdef DISPLAY
dio         EQU GPIO_GP0_POSITION    ;DIO pin of the display
clk         EQU GPIO_GP1_POSITION    ;CLK pin of the display
ir          EQU GPIO_GP2_POSITION    ;IR
#else
red         EQU GPIO_GP0_POSITION    ;Red LED
green       EQU GPIO_GP1_POSITION    ;Green LED
blue        EQU GPIO_GP2_POSITION    ;Blue LED
ir          EQU GPIO_GP3_POSITION    ;IR
#endif

; Set the configuration word.
CONFIG WDTE = OFF  ; Watchdog Timer (WDT disabled)
CONFIG CP = OFF    ; Code Protect (Code protection off)
CONFIG MCLRE = OFF ; Master Clear Enable (MCLR disabled, GP3 enabled)


; NOTE: To make sure the PIC10F200 RC oscillator calibration instruction at
; the 0xFF program memory address is not overwriten place this psect elsewhere
; , for example at the program memory start address: Project Properties >
; pic-as Global Options > Additional Options: -Wl,-pMyCode=0h
    PSECT MyCode,class=CODE,delta=2
INIT:
    MOVLW ~((1<<T0CS)|(1<<PSA)) ;enable GPIO2
    OPTION              ;and set timer prescaler to 256

#ifdef DISPLAY    
    MOVLW (1<<dio)|(1<<clk)|(1<<ir)
    MOVWF port          ;It's used to switch DIO/CLK pins direction
    TRIS GPIO           ;Set 'clk', 'dio', and 'ir' as inputs
#else
    MOVLW (1<<ir)
    TRIS GPIO           ;Set 'ir' as input
    MOVLW (1 << green) | (1 << red) | (1 << blue)  ;Set the white color
    MOVWF color         ;for the 'color' register
#endif

    CLRF GPIO           ;Clear GPIO to set all pins to 0

LOOP:                   ;Main loop of the program
;---------------Wait for the preamble positive pulse---------------------
    BTFSC GPIO, ir      ;Wait while 'ir' pin goes down
    GOTO LOOP           ;If it's high then return to 'LOOP'
    CLRF TMR0           ;Otherwise clear the timer register
    BTFSS GPIO, ir      ;And wait while 'ir' is low
    GOTO $-1
    MOVF TMR0, W        ;Copy the TMR0 value into the W register
    MOVWF count         ;and save the value into the 'count' register
    MOVLW 30            ;Load 30 into W (256 us x 32 = 7.7 ms)
    SUBWF count, W      ;And subtract W from 'count'
    BTFSS STATUS, C     ;If 'count' < 30 (pulse is shorter than 7.7 ms)
    GOTO LOOP           ;then return to 'LOOP'
    MOVLW 45            ;Load 45 into W (256 us x 45 = 11.5 ms)
    SUBWF count, W      ;And subtract W from 'count'
    BTFSC STATUS, C     ;If 'count' > 45 (pulse is longer than 8 ms)
    GOTO LOOP           ;then return to 'LOOP'
;---------------Check the preamble negative pulse---------------------
    CLRF TMR0           ;Otherwise clear the timer register
    BTFSC GPIO, ir      ;And wait while 'ir' is high
    GOTO $-1
    MOVF TMR0, W        ;Copy the TMR0 value into the W register
    MOVWF count         ;and save the value into the 'count' register
    MOVLW 13            ;Load 15 into W (256 us x 13 = 3.3 ms)
    SUBWF count, W      ;And subtract W from 'count'
    BTFSS STATUS, C     ;If 'count' < 13 (pulse is shorter than 3.3 ms)
    GOTO LOOP           ;then return to 'LOOP'
    MOVLW 20            ;Load 20 into W (256 us x 20 = 5.1 ms)
    SUBWF count, W      ;And subtract W from 'count'
    BTFSC STATUS, C     ;If 'count' > 20 (pulse is longer than 5.1 ms)
    GOTO LOOP           ;then return to 'LOOP'
;---------------Receive the command bytes-----------------------------
    CLRF byte_count     ;Clear the 'byte_count' register    
    MOVLW ir_data       ;Load the address of the 'ir_data' into W
    MOVWF FSR           ;and save it to the indirect pointer register
RECEIVE_BYTE:
    CLRF bit_count      ;Clear the 'bit_count' register
    CLRF INDF           ;Clear the indirectly addressed register
RECEIVE_BIT:
    RRF INDF, F         ;Shift the INDF register to the right
;---------------Receive the positive pulse of the bit-----------------
    CLRF TMR0           ;Otherwise clear the timer register
    BTFSS GPIO, ir      ;And wait while 'ir' is low
    GOTO $-1
    MOVF TMR0, W        ;Copy the TMR0 value into the W register
    MOVWF count         ;and save the value into the 'count' register
    MOVLW 1             ;Load 1 into W (256 us x 1 = 0.26 ms)
    SUBWF count, W      ;And subtract W from 'count'
    BTFSS STATUS, C     ;If 'count' < 1 (pulse is shorter than 0.26 ms)
    GOTO LOOP           ;then return to 'LOOP'
    MOVLW 3             ;Load 3 into W (256 us x 3 = 0.77 ms)
    SUBWF count, W      ;And subtract W from 'count'
    BTFSC STATUS, C     ;If 'count' > 3 (pulse is longer than 0.77 ms)
    GOTO LOOP           ;then return to 'LOOP'
;---------------Receive the negative pulse of the bit-----------------
    CLRF TMR0           ;Otherwise clear the timer register
    BTFSC GPIO, ir      ;And wait while 'ir' is high
    GOTO $-1
    MOVF TMR0, W        ;Copy the TMR0 value into the W register
    MOVWF count         ;and save the value into the 'count' register
    MOVLW 4             ;Load 5 into W (256 us x 4 = 1.1 ms)
    SUBWF count, W      ;And subtract W from 'count'
    BTFSS STATUS, C     ;If 'count' < 4 (pulse is shorter than 1.1 ms)
    GOTO NEXT_BIT       ;then go to the 'NEXT_BIT' label
    MOVLW 8             ;Load 8 into W (256 us x 8 = 2 ms)
    SUBWF count, W      ;And subtract W from 'count'
    BTFSC STATUS, C     ;If 'count' > 8 (pulse is longer than 2 ms)
    GOTO LOOP           ;then go to the 'LOOP' label
    BSF INDF, 7         ;Set the MSB of the INDF register
NEXT_BIT:
    INCF bit_count, F   ;Increment the 'bit_count' register
    BTFSS bit_count, 3  ;Check if 'bit_count' becomes 8
    GOTO RECEIVE_BIT    ;If it's not, then return to 'RECEIVE_BIT' label
    INCF byte_count, F  ;Increment the 'byte_count' register
    BTFSC byte_count, 2 ;Check if 'byte_count' becomes 4    
    GOTO CHECK_DATA     ;If it is then go to 'CHECK_DATA' label
    INCF FSR, F         ;Increment the indirect addressing pointer
    GOTO RECEIVE_BYTE   ;and go to 'RECEIVE_BYTE' label

CHECK_DATA:
    COMF ir_data+1, W   ;Negate the second received byte
    XORWF ir_data, W    ;And implement the XOR between 1st and 2nd bytes
    BTFSS STATUS, Z     ;If the result is not 0 (bytes are not equal)
    GOTO LOOP           ;Then return to the 'LOOP' label
    COMF ir_data+3, W   ;Negate the fourth received byte
    XORWF ir_data+2, W  ;And implement the XOR between 3rd and 4th bytes
    BTFSS STATUS, Z     ;If the result is not 0 (bytes are not equal)
    GOTO LOOP           ;Then return to the 'LOOP' label

#ifdef DISPLAY
;Send first command to LED display
    CALL TW_START       ;Issue Start condition
    MOVLW 0x40          ;Send command 0x40 -
    CALL TW_WRITE_BYTE  ;"Write data to display register"
    CALL TW_STOP        ;Issue Stop condition
;Send second command and data to LED display
    CALL TW_START       ;Issue Start condition
    MOVLW 0xC0          ;Send address 0xC0 -
    CALL TW_WRITE_BYTE  ;The first address of the display
    MOVF ir_data, W
    CALL SPLIT_NUM
    MOVF num_h, W
    CALL DECODE_DIGIT   ;Get the display data
    CALL TW_WRITE_BYTE  ;Write the first digit to the display
    MOVF num_l, W
    CALL DECODE_DIGIT   ;Get the display data
    CALL TW_WRITE_BYTE  ;Write the second digit to the display
    MOVF ir_data + 2, W
    CALL SPLIT_NUM
    MOVF num_h, W
    CALL DECODE_DIGIT   ;Get the display data
    CALL TW_WRITE_BYTE  ;Write the second digit to the display
    MOVF num_l, W
    CALL DECODE_DIGIT   ;Get the display data
    CALL TW_WRITE_BYTE  ;Write the fourth digit to the display
    CALL TW_STOP        ;Issue Stop condition
;Send brightness command to LED display
    CALL TW_START       ;Issue Start condition
    MOVLW 0x8F          ;Send display control byte 0x8F -
    CALL TW_WRITE_BYTE  ;"Display ON"
    CALL TW_STOP        ;Issue Stop condition
#else
    MOVLW 0x09          ;Check the R button (code 0x09)
    XORWF ir_data+2, W  ;If command is not 0x09
    BTFSS STATUS, Z
    GOTO $+4            ;then skip the next three lines
    MOVLW (1 << red)    ;Otherwise set the 'red' bit
    MOVWF color         ;of the 'color' register
    GOTO SET_OUTPUT     ;and go to the 'SET_OUTPUT' label

    MOVLW 0x08          ;Check the G button (code 0x08)
    XORWF ir_data+2, W  ;If command is not 0x08
    BTFSS STATUS, Z
    GOTO $+4            ;then skip the next three lines
    MOVLW (1 << green)  ;Otherwise set the 'green' bit
    MOVWF color         ;of the 'color' register
    GOTO SET_OUTPUT     ;and go to the 'SET_OUTPUT' label

    MOVLW 0x0A          ;Check the B button (code 0x0A)
    XORWF ir_data+2, W  ;If command is not 0x0A
    BTFSS STATUS, Z
    GOTO $+4            ;then skip the next three lines
    MOVLW (1 << blue)   ;Otherwise set the 'blue' bit
    MOVWF color         ;of the 'color' register
    GOTO SET_OUTPUT     ;and go to the 'SET_OUTPUT' label

    MOVLW 0x0B          ;Check the W button (code 0x0B)
    XORWF ir_data+2, W  ;If command is not 0x0B
    BTFSS STATUS, Z
    GOTO $+4            ;then skip the next three lines
    MOVLW (1 << blue) | (1 << red) | (1 << green);Otherwise set all bits
    MOVWF color         ;of the 'color' register
    GOTO SET_OUTPUT     ;and go to the 'SET_OUTPUT' label

    MOVLW 0x11          ;Check the Yellow button (code 0x11)
    XORWF ir_data+2, W  ;If command is not 0x11
    BTFSS STATUS, Z
    GOTO $+4            ;then skip the next three lines
    MOVLW (1 << green) | (1 << red)    ;Otherwise set the 'green' and 'red' bits
    MOVWF color         ;of the 'color' register
    GOTO SET_OUTPUT     ;and go to the 'SET_OUTPUT' label

    MOVLW 0x14          ;Check the Cyan button (code 0x14)
    XORWF ir_data+2, W  ;If command is not 0x14
    BTFSS STATUS, Z
    GOTO $+4            ;then skip the next three lines
    MOVLW (1 << green) | (1 << blue);Otherwise set the 'green' and 'blue' bits
    MOVWF color         ;of the 'color' register
    GOTO SET_OUTPUT     ;and go to the 'SET_OUTPUT' label

    MOVLW 0x12          ;Check the Magenta button (code 0x12)
    XORWF ir_data+2, W  ;If command is not 0x12
    BTFSS STATUS, Z
    GOTO $+4            ;then skip the next three lines
    MOVLW (1 << blue) | (1 << red)    ;Otherwise set the 'blue' and 'red' bits
    MOVWF color         ;of the 'color' register
    GOTO SET_OUTPUT     ;and go to the 'SET_OUTPUT' label

    MOVLW 0x07          ;Check the ON button (code 0x07)
    XORWF ir_data+2, W  ;If command is not 0x07
    BTFSS STATUS, Z
    GOTO $+2            ;then skip the next line
    GOTO SET_OUTPUT     ;and go to the 'SET_OUTPUT' label

    MOVLW 0x06          ;Check the OFF button (code 0x07)
    XORWF ir_data+2, W  ;If command is not 0x07
    BTFSS STATUS, Z
    GOTO $+2            ;then skip the next line
    CLRF GPIO           ;Clear the GPIO register
    GOTO LOOP           ;and go to the 'SET_OUTPUT' label

SET_OUTPUT:
    MOVF color, W       ;Copy the value of the 'color' register
    MOVWF GPIO          ;into the GPIO register
#endif

    GOTO LOOP           ;loop forever

#ifdef DISPLAY
;-------------Helper subroutines---------------
DIO_HIGH:               ;Set DIO pin high
    BSF port, dio       ;Set 'dio' bit in the 'port' to make it input
    MOVF port, W        ;Copy 'port' into W register
    TRIS GPIO           ;And set it as TRISGPIO value
    RETLW 0

DIO_LOW:                ;Set DIO pin low
    BCF port, dio       ;Reset 'dio' bit in the 'port' to make it output
    MOVF port, W        ;Copy 'port' into W register
    TRIS GPIO           ;And set it as TRISGPIO value
    RETLW 0

CLK_HIGH:               ;Set CLK pin high
    BSF port, clk       ;Set 'clk' bit in the 'port' to make it input
    MOVF port, W        ;Copy 'port' into W register
    TRIS GPIO           ;And set it as TRISGPIO value
    RETLW 0

CLK_LOW:                ;Set CLK pin low
    BCF port, clk       ;Reset 'clk' bit in the 'port' to make it output
    MOVF port, W        ;Copy 'port' into W register
    TRIS GPIO           ;And set it as TRISGPIO value
    RETLW 0
;-------------TW start condition--------------
TW_START:
    CALL CLK_HIGH       ;Set CLK high
    CALL DIO_LOW        ;Then set DIO low
    RETLW 0
;-------------TW stop condition---------------
TW_STOP:
    CALL DIO_LOW        ;Set DIO low
    CALL CLK_HIGH       ;Set CLK high
    CALL DIO_HIGH       ;Then set DIO highs and release the bus
    RETLW 0
;------------TW write byte--------------------
TW_WRITE_BYTE:
    MOVWF tw_data       ;Load 'tw_data' from W register
    MOVLW 8             ;Load value 8 into 'bit_count'
    MOVWF bit_count     ;to indicate we're going to send 8 bits
TW_WRITE_BIT:           ;Write single bit to TW
    CALL CLK_LOW        ;Set CLK low, now we can change DIO
    BTFSS tw_data, 0    ;Check the LSB of 'tw_data'
    GOTO TW_WRITE_0     ;If it's 0 then go to the 'TW_WRITE_0' label
TW_WRITE_1:             ;Else continue with 'TW_WRITE_1'
    CALL DIO_HIGH       ;Set DIO high
    GOTO TW_SHIFT       ;And go to the 'TW_SHIFT' label
TW_WRITE_0:
    CALL DIO_LOW        ;Set DIO low
TW_SHIFT:
    CALL CLK_HIGH       ;Set CLK high to start the new pulse
    RRF tw_data, F      ;Shift 'tw_data' at one bit to the right
    DECFSZ bit_count, F ;Decrement the 'bit_count' value, check if it's 0
    GOTO TW_WRITE_BIT   ;If not then return to the 'TW_WRITE_BIT'
TW_CHECK_ACK:           ;Else check the acknowledgement bit
    CALL CLK_LOW        ;Set TW low to end the last pulse
    CALL DIO_HIGH       ;Set DIO high to release the bus
    CALL CLK_HIGH       ;Set TW high to start the new pulse
    MOVF GPIO, W        ;Copy the GPIO register value into the 'ack'
    MOVWF ack           ;Now bit 'dio' of the 'ack' will contain ack bit
    CALL CLK_LOW        ;Set CLK low to end the acknowledgement bit
    RETLW 0
;------------Split number-----------------
SPLIT_NUM:
    MOVWF num_h         ;Save the W into the 'num_h'
    MOVWF num_l         ;Save the W into the 'num_l'
    SWAPF num_h, F      ;Swap the 'num_h' nibbles
    MOVLW 0x0F          ;Load 0x0F into the W register
    ANDWF num_h, F      ;Implement AND between 'num_h' and 0x0F
    ANDWF num_l, F      ;Implement AND between 'num_l' and 0x0F
    RETLW 0             ;And return from the subroutine
;------------Decode digit----------------------
DECODE_DIGIT:
    MOVWF digit         ;Copy W into 'digit'
    MOVF digit, F       ;Check if 'digit' is 0
    BTFSC STATUS, Z     ;If yes, then return with the code
    RETLW 00111111B     ;to display '0' digit
    DECFSZ digit, F     ;Decrement the 'digit' and check if it's 0
    GOTO $+2            ;If not, skip the next line
    RETLW 00000110B     ;Otherwise return with the code of '1'
    DECFSZ digit, F     ;Decrement the 'digit' and check if it's 0
    GOTO $+2            ;If not, skip the next line
    RETLW 01011011B     ;Otherwise return with the code of '2'
    DECFSZ digit, F     ;Decrement the 'digit' and check if it's 0
    GOTO $+2            ;If not, skip the next line
    RETLW 01001111B     ;Otherwise return with the code of '3'
    DECFSZ digit, F     ;Decrement the 'digit' and check if it's 0
    GOTO $+2            ;If not, skip the next line
    RETLW 01100110B     ;Otherwise return with the code of '4'
    DECFSZ digit, F     ;Decrement the 'digit' and check if it's 0
    GOTO $+2            ;If not, skip the next line
    RETLW 01101101B     ;Otherwise return with the code of '5'
    DECFSZ digit, F     ;Decrement the 'digit' and check if it's 0
    GOTO $+2            ;If not, skip the next line
    RETLW 01111101B     ;Otherwise return with the code of '6'
    DECFSZ digit, F     ;Decrement the 'digit' and check if it's 0
    GOTO $+2            ;If not, skip the next line
    RETLW 00000111B     ;Otherwise return with the code of '7'
    DECFSZ digit, F     ;Decrement the 'digit' and check if it's 0
    GOTO $+2            ;If not, skip the next line
    RETLW 01111111B     ;Otherwise return with the code of '8'
    DECFSZ digit, F     ;Decrement the 'digit' and check if it's 0
    GOTO $+2            ;If not, skip the next line
    RETLW 01101111B     ;Otherwise return with the code of '9'
    DECFSZ digit, F     ;Decrement the 'digit' and check if it's 0
    GOTO $+2            ;If not, skip the next line
    RETLW 01110111B     ;Otherwise return with the code of 'A'
    DECFSZ digit, F     ;Decrement the 'digit' and check if it's 0
    GOTO $+2            ;If not, skip the next line
    RETLW 01111100B     ;Otherwise return with the code of 'B'
    DECFSZ digit, F     ;Decrement the 'digit' and check if it's 0
    GOTO $+2            ;If not, skip the next line
    RETLW 00111001B     ;Otherwise return with the code of 'C'
    DECFSZ digit, F     ;Decrement the 'digit' and check if it's 0
    GOTO $+2            ;If not, skip the next line
    RETLW 01011110B     ;Otherwise return with the code of 'D'
    DECFSZ digit, F     ;Decrement the 'digit' and check if it's 0
    GOTO $+2            ;If not, skip the next line
    RETLW 01111001B     ;Otherwise return with the code of 'E'
    DECFSZ digit, F     ;Decrement the 'digit' and check if it's 0
    GOTO $+2            ;If not, skip the next line
    RETLW 01110001B     ;Otherwise return with the code of 'F'
    RETLW 0x00          ;In any other case return 0
#endif

    END INIT        ; Program entry point.